#include <ESP8266WiFi.h>
#include <Wire.h>
#include <U8g2lib.h>
#include "weathericons.h"

U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);

// Pas deze constanten aan (de aanhalingstekens laten staan, alleen de waarden daarbinnen wijzigen).
const char* ssid = "SSID";            // De naam van het wifi-netwerk.
const char* password = "WACHTWOORD";  // Het wachtwoord van het wifi-netwerk.
const String lat = "52.36";           // De breedtegraad van je eigen locatie (gebruik twee decimalen).
const String lon = "4.85";            // De lengtegraad van je eigen locatie (gebruik twee decimalen).
const String weerstation = "6240";    // De code van het weerstation, zie https://arduino.reshift.nl/weerslag.php voor de volledige lijst.
const String source = "weermonitor";  // Wijzig "weermonitor" in "meteoserver" voor Meteo Server, "weerlive" voor Weerlive of "weerslag" voor weerstations buiten Nederland.
const String regio = "noord-holland"; // De naam van de regio (gebruik kleine letters), zie https://arduino.reshift.nl/weeralarm.php voor de volledige lijst. 
                                      // Gebruik voor weeralarmen in België de regio "zeeland", "noord-brabant" of "limburg".
const String apikey = "";             // Alleen voor weergegevens van Weerlive is een (gratis) api-key vereist die je kunt aanvragen op https://weerlive.nl. Anders leeg laten.

// Deze constanten en variabelen kun je zo laten staan.
const String rainurl = "/data/raintext/?lat=" + lat + "&lon=" + lon;
const String alarmurl = "/weeralarm.php?regio=" + regio;
const String weerurl = "/" + source + ".php?weerstation=" + weerstation + "&key=" + apikey;
const int wit = 16; // Pin D0
const int buzzer = 14; // Pin D5
const int blauw = 2; // Pin D4
const int groen = 0; // Pin D3
const int geel = 12; // Pin D6
const int oranje = 13; // Pin D7
const int rood = 15; // Pin D8
const int wachttijd = 300000; // 5 minuten.
const int periode = 1; // Aantal perioden van 5 minuten vooruit, minimumaantal is 1.
const unsigned long interval = 3600000; // Tijd (in milliseconden) dat het zeker stil blijft na een keer piepen.
unsigned long previousMillis = 0;
const char* weerhost = "arduino.reshift.nl";
const int httpPort = 80;
bool nat = false;
bool ijs = false;
bool alarm = false;
bool eerstepiep = true;
const String code_rood="warning-overview--red";
const String code_oranje="warning-overview--orange";
const String code_geel="warning-overview--yellow";
const String code_groen="warning-overview--green";
const String params_url = "/nu.php";
const String tijd_begin_tag = "<tijdstip>";
const String tijd_einde_tag = "</tijdstip>";
String nu = "";
String weersituatie = "";
int fouten = 0;
bool verbindingsprobleem = false;

WiFiClient client;

void setup() {
  u8g2.begin();
  Serial.begin(115200);
  pinMode(wit, OUTPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(blauw, OUTPUT);
  pinMode(groen, OUTPUT);
  pinMode(geel, OUTPUT);
  pinMode(oranje, OUTPUT);
  pinMode(rood, OUTPUT);
  digitalWrite(blauw, LOW);
  digitalWrite(groen, LOW);
  digitalWrite(geel, LOW);
  digitalWrite(oranje, LOW);
  digitalWrite(rood, LOW);
  digitalWrite(buzzer, LOW);

  // Verbind met WiFi-netwerk
  connectWifi();
  
  Serial.print("Verbind met ");
  Serial.println(weerhost);

  if (!client.connect(weerhost, httpPort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    return;
  }

  Serial.print(weerhost);
  Serial.println(" bereikbaar");
  Serial.println();
  Serial.println("Verbreek verbinding");
  client.stop();

  digitalWrite(wit, HIGH);
  delay(500);
  digitalWrite(wit, LOW);
  digitalWrite(blauw, HIGH);
  delay(500);
  digitalWrite(blauw, LOW);
  digitalWrite(groen, HIGH);
  delay(500);
  digitalWrite(groen, LOW);
  digitalWrite(geel, HIGH);
  delay(500);
  digitalWrite(geel, LOW);
  digitalWrite(oranje, HIGH);
  delay(500);
  digitalWrite(oranje, LOW);
  digitalWrite(rood, HIGH);
  delay(500);
  digitalWrite(rood, LOW);
  tone(buzzer, 1000);
  delay(500);
  noTone(buzzer);

}

void connectWifi() {
  // Verbind met WiFi-netwerk
  int i = 0;
  WiFi.mode(WIFI_STA); // De ESP is geen AP
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    i++;
    if (i >= 120) {
      ESP.restart();
    }
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());
}

void piepen() {
  unsigned long currentMillis = millis();
  // Onderstaande blijft ook correct werken als millis() na 49 dagen weer bij 0 begint (roll-over).
  if ((unsigned long)(currentMillis - previousMillis) >= interval || eerstepiep == true) {
    tone(buzzer, 4000);
    delay(100);
    noTone(buzzer); 
    delay(100);
    tone(buzzer, 4000);
    delay(100);
    noTone(buzzer); 
    previousMillis = currentMillis;
    eerstepiep = false;
  }
}

void displayweather(const uint8_t* winddir, String temp, String wind, const uint8_t* symbol) {
    wind += " Bft";
    temp += " ";
    temp += ((char)176);
    temp += "C";
    u8g2.clearDisplay();
    u8g2.firstPage();
    do {
      u8g2.drawXBM(0,0,50,50,symbol);
      u8g2.drawXBM(75,0,50,50,winddir);
      u8g2.setFont(u8g2_font_helvB12_tf);
      u8g2.setCursor(2, 64);
      u8g2.print(temp);
      u8g2.setCursor(80, 64);
      u8g2.print(wind);
      // Inverteer beeld tussen 07.00 uur en 19.00 uur.
      if(nu.startsWith("07") || nu.startsWith("08") || nu.startsWith("09") || nu.startsWith("10") || nu.startsWith("11") || nu.startsWith("12") || nu.startsWith("13") || nu.startsWith("14") || nu.startsWith("15") || nu.startsWith("16") || nu.startsWith("17") || nu.startsWith("18")) {
        u8g2.sendF("c", 0x0a7);
      }
      else {
        u8g2.sendF("c", 0x0a6);
      }
    } while( u8g2.nextPage() );
  delay(1000);
}

void lees_tot_tag(String begin_tag, String einde_tag, int vorst = 0) {
  String inhoud = "";
  while(client.available() && inhoud==""){
    String regel = client.readStringUntil('\r');
    int begin_pos = regel.indexOf(begin_tag);
    if(begin_pos != -1) {
      int einde_pos = regel.indexOf(einde_tag);
      inhoud = regel.substring(begin_pos + begin_tag.length(), einde_pos);
      weersituatie += inhoud += ",";
    }
  }
  if (vorst != 0 && inhoud.indexOf('-') != -1 && inhoud.length() > 2) {
    digitalWrite(wit, HIGH);
    if (ijs == false) {
      piepen();
      ijs = true;
    }
  }
  else if (vorst!= 0) {
    digitalWrite(wit, LOW);
    ijs = false;
  }
}

void checkalarm() {

  int gevonden = 0;
  Serial.print("Verbind met ");
  Serial.println(weerhost);
  
  if (!client.connect(weerhost, httpPort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    fouten++;
    return;
  }

  Serial.print("Vraag URL op: ");
  Serial.println(alarmurl);

  client.print(String("GET ") + alarmurl + " HTTP/1.1\r\n" +
                 "Host: " + weerhost + "\r\n\r\n");
               
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println("Verbinding mislukt: timeout");
      client.stop();
      return;
    }
  }

  while(client.available() && gevonden == 0){
    String regel = client.readStringUntil('\r');
    if(regel.indexOf(code_rood) != -1) {
      digitalWrite(rood, HIGH);
      digitalWrite(oranje, LOW);
      digitalWrite(geel, LOW);
      digitalWrite(groen, LOW);
      if(alarm == false){
        piepen();
        alarm = true;
      }
      gevonden = 1;
    } else if(regel.indexOf(code_oranje) != -1) {
      digitalWrite(oranje, HIGH);
      digitalWrite(rood, LOW);
      digitalWrite(geel, LOW);
      digitalWrite(groen, LOW);
      if(alarm == false){
        piepen();
        alarm = true;
      }
      gevonden = 1;
    } else if(regel.indexOf(code_geel) != -1) {
      digitalWrite(geel, HIGH);
      digitalWrite(rood, LOW);
      digitalWrite(oranje, LOW);
      digitalWrite(groen, LOW);
      if(alarm == false){
        piepen();
        alarm = true;
      }
      gevonden = 1;
    } else if(regel.indexOf(code_groen) != -1) {
      digitalWrite(groen, HIGH);
      digitalWrite(rood, LOW);
      digitalWrite(oranje, LOW);
      digitalWrite(geel, LOW);
      alarm = false;
      gevonden = 1;
    } 
  }

  Serial.println();
  Serial.println("Verbreek verbinding");
  client.stop();
}

void checkweer() {
  int weerstation_gevonden = 0;
  bool tonen = false;
  weersituatie = "";

  Serial.print("Verbind met ");
  Serial.println(weerhost);
  
  if (!client.connect(weerhost, httpPort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    fouten++;
    return;
  }

  Serial.print("Vraag URL op: ");
  Serial.println(weerurl);

  client.print(String("GET ") + weerurl + " HTTP/1.1\r\n" +
                 "Host: " + weerhost + "\r\n\r\n");
               
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println("Verbinding mislukt: timeout");
      client.stop();
      return;
    }
    tonen = true;
  }

  while(client.available() && weerstation_gevonden==0){
    String regel = client.readStringUntil('\r');
    if(regel.indexOf("<stationcode>" + weerstation) != -1){
      weerstation_gevonden = 1;      
    }
  }

  if(weerstation_gevonden==0) {
    Serial.println();
	Serial.println("Weerstation " + weerstation + " niet gevonden");
  } else {
    lees_tot_tag("<temperatuurGC>", "</temperatuurGC>", 1);
    lees_tot_tag("<windsnelheidBF>", "</windsnelheidBF>");
    lees_tot_tag("<windrichting>", "</windrichting>");
    lees_tot_tag("<icoonactueel ID=\"", "\" zin=");
    Serial.println();
    Serial.print("Weersituatie: ");

    weersituatie += "EOF";
    String temperatuur = getValue(weersituatie, ',', 0);
    String windkracht = getValue(weersituatie, ',', 1);
    String windrichting = getValue(weersituatie, ',', 2);
    String pictogram = getValue(weersituatie, ',', 3);
    Serial.println(temperatuur);
    Serial.println(windkracht);
    Serial.println(windrichting);
    Serial.println(pictogram);
    const uint8_t* winddir;
    const uint8_t* icon;

    if(windrichting == "N"){
        winddir = N_bits;
    }
    else if(windrichting == "NNW"){
        winddir = NNW_bits;
    }
    else if(windrichting == "NW"){
        winddir = NW_bits;
    }
    else if(windrichting == "WNW"){
        winddir = WNW_bits;
    }
    else if(windrichting == "W"){
        winddir = W_bits;
    }
    else if(windrichting == "WZW"){
        winddir = WSW_bits;
    }
    else if(windrichting == "ZW"){
        winddir = SW_bits;
    }
    else if(windrichting == "ZZW"){
        winddir = SSW_bits;
    }
    else if(windrichting == "Z"){
        winddir = S_bits;
    }
    else if(windrichting == "ZZO"){
        winddir = SSE_bits;
    }
    else if(windrichting == "ZO"){
        winddir = SE_bits;
    }
    else if(windrichting == "OZO"){
        winddir = ESE_bits;
    }
    else if(windrichting == "O"){
        winddir = E_bits;
    }
    else if(windrichting == "ONO"){
        winddir = ENE_bits;
    }
    else if(windrichting == "NO"){
        winddir = NE_bits;
    }
    else if(windrichting == "NNO"){
        winddir = NNE_bits;
    }

    if(pictogram == "a"){
        icon = a_bits;
    }
    else if(pictogram == "b" || pictogram == "o"){
        icon = b_bits;
    }
    else if(pictogram == "c" || pictogram == "cc" || pictogram == "p" || pictogram == "pp"){
        icon = c_bits;
    }
    else if(pictogram == "d"){
        icon = d_bits;
    }
    else if(pictogram == "e" || pictogram == "ee"){
        icon = e_bits;
    }
    else if(pictogram == "f" || pictogram == "k"){
        icon = f_bits;
    }
    else if(pictogram == "g"){
        icon = g_bits;
    }
    else if(pictogram == "h" || pictogram == "i"){
        icon = h_bits;
    }
    else if(pictogram == "l" || pictogram == "ll" || pictogram == "m" || pictogram == "mm" || pictogram == "q" || pictogram == "qq"){
        icon = l_bits;
    }
    else if(pictogram == "n" || pictogram == "nn"){
        icon = n_bits;
    }
    else if(pictogram == "s" || pictogram == "ss"){
        icon = s_bits;
    }
    else if(pictogram == "t" || pictogram == "tt" || pictogram == "v" || pictogram == "vv" || pictogram == "w" || pictogram == "ww" || pictogram == "x" || pictogram == "xx" || pictogram == "y" || pictogram == "yy" || pictogram == "z" || pictogram == "zz"){
        icon = t_bits;
    }
    else if(pictogram == "u"){
        icon = u_bits;
    }
    else if(pictogram == "aa"){
        icon = aa_bits;
    }
    else if(pictogram == "bb" || pictogram == "oo"){
        icon = bb_bits;
    }
    else if(pictogram == "dd"){
        icon = dd_bits;
    }
    else if(pictogram == "ff" || pictogram == "kk"){
        icon = ff_bits;
    }
    else if(pictogram == "gg"){
        icon = gg_bits;
    }
    else if(pictogram == "hh" || pictogram == "ii"){
        icon = hh_bits;
    }
    else if(pictogram == "uu"){
        icon = uu_bits;
    }

    displayweather(winddir, temperatuur, windkracht, icon); // De output voor het scherm.
  }

  Serial.println();
  Serial.println("Verbreek verbinding");
  client.stop();
}

String getValue(String data, char separator, int index)
{
    int found = 0;
    int strIndex[] = { 0, -1 };
    int maxIndex = data.length() - 1;
    for (int i = 0; i <= maxIndex && found <= index; i++) {
        if (data.charAt(i) == separator || i == maxIndex) {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i+1 : i;
        }
    }
    return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}

void checktime() {
  Serial.print("Verbind met ");
  Serial.println(weerhost);
  
  if (!client.connect(weerhost, httpPort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    fouten++;
    return;
  }

  Serial.print("Vraag URL op: ");
  Serial.println(params_url);

  client.print(String("GET ") + params_url + " HTTP/1.1\r\n" +
                 "Host: " + weerhost + "\r\n\r\n");
               
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println("Verbinding mislukt: timeout");
      client.stop();
      return;
    }
  }

  String inhoud = "";
  while(client.available() && inhoud==""){
    String regel = client.readStringUntil('\r');
    int begin_pos = regel.indexOf(tijd_begin_tag);
    if(begin_pos != -1) {
      int einde_pos = regel.indexOf(tijd_einde_tag);
      String inhoud = regel.substring(begin_pos + tijd_begin_tag.length(), einde_pos);
      nu = inhoud;
      Serial.print("Huidig tijdstip: ");
      Serial.println(nu);
    }
  }

  Serial.println();
  Serial.println("Verbreek verbinding");
  client.stop();
}

void checkrain() {
  Serial.print("Verbind met ");
  Serial.println(weerhost);
  
  if (!client.connect(weerhost, httpPort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    fouten++;
    return;
  }

  int randNumber = random(999999);
  Serial.print("Vraag URL op: ");
  Serial.println(rainurl + '&' + randNumber);
  client.print(String("GET ") + rainurl + '&' + randNumber + " HTTP/1.1\r\n" +
                 "Host: " + weerhost + "\r\n\r\n");
  client.println("Cache-Control: no-cache, no-store");
                     
  unsigned long timeout2 = millis();
  while (client.available() == 0) {
    if (millis() - timeout2 > 5000) {
      Serial.println("Verbinding mislukt: timeout");
      client.stop();
      return;
    }
  }
  
  while(client.available()){
    String page = client.readString();
    String eersteregel = getValue(page, '|', 1);
    String tweederegel = getValue(page, '|', 2);
    String derderegel = getValue(page, '|', 3);
    String regellang = getValue(page, '|', periode + 1);
    String regel = getValue(regellang, '\r', 1);
    Serial.print("Tweede periode: ");
    Serial.println(regel);
    int regen = regel.toInt();
    String regellangnu = getValue(page, '|', periode);
    String regelnu = getValue(regellangnu, '\r', 1);
    Serial.print("Eerste periode: ");
    Serial.println(regelnu);
    int regennu = regelnu.toInt();

    if(regen > 0 && regennu > 0 && (eersteregel.indexOf(nu) != -1 || tweederegel.indexOf(nu) != -1 || derderegel.indexOf(nu) != -1)) {
      digitalWrite(blauw, HIGH);
      if (nat == false) {
        piepen();
        nat = true;
      }
    }
    else if(regen == 0 && regennu == 0 && (eersteregel.indexOf(nu) != -1 || tweederegel.indexOf(nu) != -1 || derderegel.indexOf(nu) != -1)) {
      digitalWrite(blauw, LOW);
      nat = false;
    }
  }
  
  Serial.println();
  Serial.println("Verbreek verbinding");
  client.stop();
}

void loop() {
  fouten = 0;
  checktime();
  checkrain();
  checkweer();
  checkalarm();
  if (fouten == 4 && verbindingsprobleem == true) {
    verbindingsprobleem = false;
    WiFi.disconnect(true);
    connectWifi();
  }
  else if (fouten == 4) {
    verbindingsprobleem = true;
  }
  else {
    verbindingsprobleem = false;
  }
  delay(wachttijd);
  if (WiFi.status() != WL_CONNECTED) {
    connectWifi();
  }
}
